/*     Bed a Binary EDitor for Linux and for Rxvt running under Linux.       */
/*     Copyright (C) 1998  Jaap Korthals Altes <binaryeditor+bed@eml.cc>      */
/*                                                                           */
/*     Bed is free software; you can redistribute it and/or modify           */
/*     it under the terms of the GNU General Public License as published by  */
/*     the Free Software Foundation; either version 2 of the License, or     */
/*     (at your option) any later version.                                   */
/*                                                                           */
/*     Bed is distributed in the hope that it will be useful,                */
/*     but WITHOUT ANY WARRANTY; without even the implied warranty of        */
/*     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         */
/*     GNU General Public License for more details.                          */
/*                                                                           */
/*     You should have received a copy of the GNU General Public License     */
/*     along with bed; if not, write to the Free Software                    */
/*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             */
/*                                                                           */
/* Sun Dec  6 18:34:27 1998                                                  */

/***
 *  v2  2000.09.20  Jim Blackson	rename truncate to truncatefile.
 */

#ifndef MEMORY_H
#define MEMORY_H
#include <features.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/param.h>
#include "defines.h"
#include "rbtree.h"
#include "offtypes.h"
#ifdef __FreeBSD__
#define NEEDS_BLOCKMODE 1
#endif
#ifdef BIOSDISK
#define NEEDS_BLOCKMODE 1
#endif
#ifdef  __cplusplus
extern "C" {
#endif
#ifdef HAS_RE_SEARCH
#ifdef HAVE_GNUREGEX_H
#include <gnuregex.h>
#else
#include <regex.h>
#endif
#else
#include "regex.h"
#endif

#ifdef  __cplusplus
}
#endif
#define DEBUG DEBUGGING
#include "debug.h"
#define regsearch(mem) (((mem)->lastforward>>2)&1)
#define forwardsearch(mem) (((mem)->lastforward&1)
#define casesearching(mem) (((mem)->lastforward>>1)&1)
#define BIGEST   LONG_MAX

#if !defined(USE_PREAD) && defined(USETHREADS)
#include <semaphore.h>
#define DEFINEUSELSEEK sem_t lseeksem
#define DESTROYUSELSEEK  sem_destroy(&lseeksem)
#define INITUSELSEEK sem_init(&lseeksem,0,1)
#define STARTUSELSEEK sem_wait(&lseeksem)
#define	STOPUSELSEEK sem_post(&lseeksem)
#define	GSTOPUSELSEEK(memp) sem_post(&((memp)->lseeksem))
#else
#define	GSTOPUSELSEEK(memp)
#define DEFINEUSELSEEK 
#define DESTROYUSELSEEK  
#define INITUSELSEEK 
#define STARTUSELSEEK
#define STOPUSELSEEK 
#endif


extern int STBLOCKSIZE;


//#define DEBUG(...) fprintf(stderr,__VA_ARGS__)
//#define DEBUG(x...)
#undef minnum
#undef maxnum
#define minnum( x,y) ((x)>(y)?(y):(x))
#define maxnum( een, twee) (((een)>(twee))?(een):(twee))
typedef char Flag;
#ifdef BIOSDISK
#include "biosdisk.h"
#endif
#ifndef _WIN32
typedef int BOOL ;
#endif
class Memory {
public:
#ifdef BIOSDISK
	Diskinfo *infodisk;	
#endif
private:
#ifdef NEEDS_BLOCKMODE
	int blockreadonly;
#endif
	public:
DEFINEUSELSEEK ;
int BLOCKSIZE;
	FILEHANDLE filehandle;
	int errors,readonly;
Treel *gettreel(BLOCKTYPE blockiter) ;
	 STATTYPE statdata;

	OFFTYPE filesize, diskfilesize;
	int lastforward;

	Flag mmapfailed;
	OFFTYPE searchpos;
      struct re_pattern_buffer *searchexpr;
      struct re_registers *searchregs;
	int tablebase[256];
	int *table;
	int searchlen;
	char *searchstr;

	char *filename;
	Tree tree;
	int error(void) {return errors;}
	OFFTYPE size(void) {return filesize;}
	Memory(char *name);
	~Memory();

void openfile(void) ;
void init(char *name) ;
int reinit(char *name) ;

void endfile(void);
int reinit(void) ;
int empty(void) ;

int savenode(FILEHANDLE output, Treel *node) ;
	int saveall(void);
	int saveto(int handle);
	SIZE_T saveto(int handle,OFFTYPE beg,OFFTYPE len) ;
	int touched(void) ;
	SIZE_T getblock(BLOCKTYPE blocknr,char *buf) ;

SIZE_T mmapgetblock(BLOCKTYPE blocknr,char *buf) ;
	SIZE_T putblock(BLOCKTYPE blocknr,char *buf) ;
	SIZE_T getblockpiece(OFFTYPE beg,SIZE_T len,char *get) ;
	OFFTYPE getpart(OFFTYPE beg,OFFTYPE len,char *gegs) ;
SIZE_T putblockpiece(OFFTYPE  beg,SIZE_T len,char *get) ;
OFFTYPE putpart(OFFTYPE beg,OFFTYPE len,char *gegs) ;
void	setborder(OFFTYPE pos) ;
void extend(OFFTYPE newsize) ;
void truncatefile(OFFTYPE pos) ;			/* v2 */
//f c r

inline BLOCKTYPE blocksize(void) { return BLOCKSIZE;};
void initsearch( void) ;
OFFTYPE search(OFFTYPE beg,char *str,int len) ;
int setsearchgegs(OFFTYPE beg,char *str,int len) ;

	OFFTYPE nextsearch(void) ;

OFFTYPE backicaseregexsearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE backregexsearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextbackregexsearch(void) ;



OFFTYPE icaseregexsearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE regexsearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextforwardregexsearch(void) ;
OFFTYPE icasesearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextforwardicasesearch(void) ;
OFFTYPE casesearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextforwardsearch(void) ;
OFFTYPE backsearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextbacksearch(void) ;
OFFTYPE backicasesearch(OFFTYPE beg,char *str,int len) ;
OFFTYPE nextbackicasesearch(void) ;


SIZE_T blockreadat(char *ptr,SIZE_T size,OFFTYPE offset) ;
inline	SIZE_T readat(char *ptr,SIZE_T size,OFFTYPE offset) ;

SIZE_T writeat(FILEHANDLE output,char *ptr,SIZE_T size,OFFTYPE offset) ;
int changed(void) ;
int changednode(Treel *node) ;
OFFTYPE exploresize(FILEHANDLE file,OFFTYPE begin=0, OFFTYPE end=MAXDISCSIZE) ;

#ifdef BIOSDISK
OFFTYPE getregistrysize(Diskinfo *infodisk,OFFTYPE &begin,OFFTYPE &end) ;
BOOL setregistrysize(Diskinfo *infodisk,OFFTYPE filesize);
#endif
inline BOOL sizehigher(OFFTYPE sizefile) ;
inline BOOL testsize(OFFTYPE sizetest);
inline BOOL checkfilesize(OFFTYPE filsize,OFFTYPE &begin,OFFTYPE &end) ;
OFFTYPE win32diskfilesize(FILEHANDLE filehandle) ;
	};


extern int tmpbuffer(char *name); 
extern SIZE_T writetofile(FILEHANDLE output,char *buf,SIZE_T nr);
#endif
